<?php

namespace Drupal\Tests\responsive_image_preload\Unit;

use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\responsive_image_preload\FieldPreprocessor;
use Drupal\responsive_image_preload\PreloadGeneratorInterface;
use Drupal\Tests\UnitTestCase;

/**
 * Test cases for the field preprocessor service.
 *
 * @group responsive_image_preload
 */
class FieldPreprocessorTest extends UnitTestCase {

  /**
   * The subject under test.
   *
   * @var \Drupal\responsive_image_preload\FieldPreprocessor
   */
  protected $instance;

  /**
   * The mocked display settings.
   *
   * @var \Drupal\Core\Entity\Display\EntityViewDisplayInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $display;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $preload_generator = $this->getMockBuilder(PreloadGeneratorInterface::class)
      ->disableOriginalConstructor()
      ->getMock();

    $preload_generator
      ->method('generatePreloads')
      ->willReturn([
        'test-preload-key' => ['test-preload-data'],
      ]);

    $this->display = $this->getMockBuilder(EntityViewDisplayInterface::class)
      ->disableOriginalConstructor()
      ->getMock();

    $storage = $this->getMockBuilder(EntityStorageInterface::class)
      ->disableOriginalConstructor()
      ->getMock();

    $storage->method('load')
      ->willReturn($this->display);

    $entity_type_manager = $this->getMockBuilder(EntityTypeManagerInterface::class)
      ->disableOriginalConstructor()
      ->getMock();

    $entity_type_manager->method('getStorage')
      ->willReturn($storage);

    $this->instance = new FieldPreprocessor($preload_generator, $entity_type_manager);
  }

  /**
   * Sets the enabled state of the 'generate_preloads' third party setting.
   *
   * @param bool $enabled
   *   The new state of the 'generate_preloads' third party setting.
   */
  protected function setPreloadsEnabled($enabled) {
    $this->display
      ->method('getComponent')
      ->willReturn([
        'third_party_settings' => [
          'responsive_image_preload' => [
            'generate_preloads' => $enabled,
          ],
        ],
      ]
      );
  }

  /**
   * Gets a mocked entity instance.
   *
   * @return \Drupal\Core\Entity\FieldableEntityInterface|\PHPUnit\Framework\MockObject\MockObject
   *   A mocked entity instance.
   */
  protected function getMockEntity() {
    $entity = $this->getMockBuilder(FieldableEntityInterface::class)
      ->disableOriginalConstructor()
      ->getMock();

    $entity->method('getEntityTypeId')
      ->willReturn('test');
    $entity->method('bundle')
      ->willReturn('test');

    return $entity;
  }

  /**
   * Test case for when an unsupported formatter is provided.
   */
  public function testFieldPreprocessorWrongFormatter() {
    $variables = [
      'element' => [
        '#formatter' => 'not-the-right-one',
      ],
    ];

    $this->instance->preprocessField($variables);

    static::assertArrayNotHasKey('#attached', $variables);
  }

  /**
   * Test case for when preload generation is disabled.
   */
  public function testFieldPreprocessorPreloadsTurnedOff() {

    $variables = [
      'element' => [
        '#formatter' => 'responsive_image',
        '#object' => $this->getMockEntity(),
        '#view_mode' => 'test',
        '#field_name' => 'test',
      ],
    ];

    $this->setPreloadsEnabled(FALSE);

    $this->instance->preprocessField($variables);

    static::assertArrayNotHasKey('#attached', $variables);

  }

  /**
   * Test case for when preload generation is enabled.
   */
  public function testFieldPreprocessorTurnedOn() {

    $variables = [
      'element' => [
        '#formatter' => 'responsive_image',
        '#object' => $this->getMockEntity(),
        '#view_mode' => 'test',
        '#field_name' => 'test',
      ],
    ];

    $this->setPreloadsEnabled(TRUE);

    $this->instance->preprocessField($variables);

    $expected = [
      'html_head' => [[
        ['test-preload-data'],
        'test-preload-key',
      ],
      ],
    ];
    static::assertSame($expected, $variables['#attached']);
  }

}
